/***********************************************************************************************************************
135MMBasic

IOPorts.h

Include file that defines the IOPins for the PIC32 chip in MMBasic.

Copyright 2011 - 2015 Geoff Graham.  All Rights Reserved.

This file and modified versions of this file are supplied to specific individuals or organisations under the following
provisions:

- This file, or any files that comprise the MMBasic source (modified or not), may not be distributed or copied to any other
  person or organisation without written permission.

- Object files (.o and .hex files) generated using this file (modified or not) may not be distributed or copied to any other
  person or organisation without written permission.

- This file is provided in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

************************************************************************************************************************/

#ifndef IOPORTS_H
#define IOPORTS_H

// these are the valid peek/poke memory ranges for the STM32F407VG
#define PEEKRANGE(a) ((a >= RAMBASE && a <= RAMEND) || (a >= 0x40000000 && a <= 0x4007FFFF) || (a >= 0x10000000 && a <= 0x1000FFFF)|| (a >= 0x8000000 && a <= 0x800FFFFF))
#define POKERANGE(a) ((a >= RAMBASE && a <= RAMEND) || (a >= 0x10000000 && a <= 0x1000FFFF) || (a >= 0x40000000 && a <= 0x4007FFFF))

// General defines
#define P_INPUT				1						// for setting the TRIS on I/O bits
#define P_OUTPUT			0
#define P_ON				1
#define P_OFF				0
// Structure that defines the SFR, bit number and mode for each I/O pin
struct s_PinDef {
	GPIO_TypeDef *sfr;
    unsigned int bitnbr;
    unsigned char mode;
#ifdef STM32H753xx
    ADC_TypeDef *  ADC;
#else
    ADC_TypeDef *  ADCn;
#endif
    uint32_t ADCchannel;
};
typedef struct s_PinDef PinDefAlias;

// Defines for the various modes that an I/O pin can be set to
#define PUNUSED       1
#define ANALOG_IN    2
#define DIGITAL_IN   4
#define COUNTING     8
#define INTERRUPT    16
#define DIGITAL_OUT  32
#define OC_OUT       64
#define DO_NOT_RESET 128
//#define HAS_64PINS 0
#define NBR_PINS_144CHIP    144					    // number of pins for external i/o on a 144 pin chip
#define NBR_PINS_100CHIP    100					    // number of pins for external i/o on a 100 pin chip
#define NBR_PINS_64CHIP    64					    // number of pins for external i/o on a 64 pin chip
#ifndef STM32F4port
#define NBRPINS             144
#else
#define package (*(volatile unsigned int *)(PACKAGE_BASE) & 0b11111)
#define flashsize *(volatile unsigned int *)(FLASHSIZE_BASE)
#define chipID (DBGMCU->IDCODE & 0x00000FFF)
#define HAS_32PINS          (package==0x8)
#define HAS_48PINS          (package==0xB)
#define HAS_64PINS          (package==0x9 || package==0)
#define HAS_100PINS          (package==0x10)
#define NBRPINS             (HAS_64PINS ? 64 : 100)
#endif

#define NBRINTPINS          10
#if defined(DEFINE_PINDEF_TABLE)
#ifndef STM32F4port
const struct s_PinDef PinDef144[NBR_PINS_144CHIP + 1]={
	    { NULL,  0, PUNUSED , NULL, 0},                                                       		// pin 0
	    { GPIOE,  GPIO_PIN_2,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},            	 				// pin 1 SSD_D2
	    { GPIOE,  GPIO_PIN_3,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},            	 				// pin 2 SSD_D3
	    { GPIOE,  GPIO_PIN_4,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},            	 				// pin 3 SSD_D4
	    { GPIOE,  GPIO_PIN_5,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},            	 				// pin 4 SSD_D5
	    { GPIOE,  GPIO_PIN_6,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},            	 				// pin 5 SSD_D6
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 6 VBAT
	    { GPIOC,  GPIO_PIN_13,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},           	 				// pin 7 - Push Button
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 8 OSC32-IN
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 9 OSC32-OUT
	    { GPIOF,  GPIO_PIN_0, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 10 COUNT1
	    { GPIOF,  GPIO_PIN_1,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},            	 				// pin 11 COUNT2
	    { GPIOF,  GPIO_PIN_2,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},            	 				// pin 12 COUNT3
#ifdef STM32H753xx
	    { GPIOF,  GPIO_PIN_3,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_5},     // pin 13 COUNT4
	    { GPIOF,  GPIO_PIN_4,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_9},     // pin 14 IR
	    { GPIOF,  GPIO_PIN_5,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_4},     // pin 15
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 16 VSS
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 17 VDD
		{ GPIOF,  GPIO_PIN_6,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_8},     // pin 18
		{ GPIOF,  GPIO_PIN_7,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_3},     // pin 19 SPI5-CLK
		{ GPIOF,  GPIO_PIN_8,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_7},     // pin 20 SPI5-IN
		{ GPIOF,  GPIO_PIN_9,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_2},     // pin 21 SPI5-OUT
	    { GPIOF,  GPIO_PIN_10,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_6},    // pin 22
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 23 RCC-OSCIN
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 24 RCC-OSCOUT
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 25 NRST
	    { GPIOC,  GPIO_PIN_0,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_10},    // pin 26 OV7670_DAT0
	    { GPIOC,  GPIO_PIN_1,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_11},    // pin 27 SPI2-OUT, OV7670_DAT1
	    { GPIOC,  GPIO_PIN_2, DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_0},      // pin 28 SPI2-IN, OV7670_DAT2
	    { GPIOC,  GPIO_PIN_3, DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_1},      // pin 29 OV7670_DAT3
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 30 VDD
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 31 VSSA
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 32 VREF+
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 33 VDDA
	    { GPIOA,  GPIO_PIN_0,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_16},    // pin 34 PWM-2A
	    { GPIOA,  GPIO_PIN_1,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_17},    // pin 35 PWM-2B
	    { GPIOA,  GPIO_PIN_2,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_14},    // pin 36 PWM-2C
	//
		{ GPIOA,  GPIO_PIN_3,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_15},    // pin 37 PWM-2D
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 38 VSS
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 39 VDD
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 40 DAC1
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 41 DAC2
		{ GPIOA,  GPIO_PIN_6,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_3},     // pin 42 SPI-IN
		{ GPIOA,  GPIO_PIN_7,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_7},     // pin 43 SPI-OUT
	    { GPIOC,  GPIO_PIN_4,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_4},     // pin 44 OV7670_DAT4
	    { GPIOC,  GPIO_PIN_5, DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_8},      // pin 45 OV7670_DAT5
	    { GPIOB,  GPIO_PIN_0, DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_9},      // pin 46 Green-LED
	    { GPIOB,  GPIO_PIN_1,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_5},     // pin 47
	    { GPIOB,  GPIO_PIN_2,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},            	 				// pin 48 SPI3-OUT
	    { GPIOF,  GPIO_PIN_11,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_2},    // pin 49
	    { GPIOF,  GPIO_PIN_12,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_6},    // pin 50
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 51 VSS
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 52 VDD
		{ GPIOF,  GPIO_PIN_13,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN ,ADC2, ADC_CHANNEL_2},     // pin 53
		{ GPIOF,  GPIO_PIN_14,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN ,ADC2, ADC_CHANNEL_6},     // pin 54
#else
	    { GPIOF,  GPIO_PIN_3,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_9},     // pin 13 COUNT4
	    { GPIOF,  GPIO_PIN_4,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_14},    // pin 14 IR
	    { GPIOF,  GPIO_PIN_5,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_15},    // pin 15
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 16 VSS
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 17 VDD
		{ GPIOF,  GPIO_PIN_6,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_4},     // pin 18
		{ GPIOF,  GPIO_PIN_7,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_5},     // pin 19 SPI5-CLK
		{ GPIOF,  GPIO_PIN_8,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_6},     // pin 20 SPI5-IN
		{ GPIOF,  GPIO_PIN_9,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_7},     // pin 21 SPI5-OUT
	    { GPIOF,  GPIO_PIN_10,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC3, ADC_CHANNEL_8},    // pin 22
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 23 RCC-OSCIN
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 24 RCC-OSCOUT
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 25 NRST
	    { GPIOC,  GPIO_PIN_0,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC2, ADC_CHANNEL_10},    // pin 26 OV7670_DAT0
	    { GPIOC,  GPIO_PIN_1,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC2, ADC_CHANNEL_11},    // pin 27 SPI2-OUT, OV7670_DAT1
	    { GPIOC,  GPIO_PIN_2, DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC2, ADC_CHANNEL_12},     // pin 28 SPI2-IN, OV7670_DAT2
	    { GPIOC,  GPIO_PIN_3, DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC2, ADC_CHANNEL_13},      // pin 29 OV7670_DAT3
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 30 VDD
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 31 VSSA
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 32 VREF+
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 33 VDDA
	    { GPIOA,  GPIO_PIN_0,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_0},     // pin 34 PWM-2A
	    { GPIOA,  GPIO_PIN_1,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_1},     // pin 35 PWM-2B
	    { GPIOA,  GPIO_PIN_2,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_2},     // pin 36 PWM-2C
	//
		{ GPIOA,  GPIO_PIN_3,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_3},     // pin 37 PWM-2D
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 38 VSS
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 39 VDD
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 40 DAC1
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 41 DAC2
		{ GPIOA,  GPIO_PIN_6,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_6},     // pin 42 SPI-IN
		{ GPIOA,  GPIO_PIN_7,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_7},     // pin 43 SPI-OUT
	    { GPIOC,  GPIO_PIN_4,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_14},    // pin 44 OV7670_DAT4
	    { GPIOC,  GPIO_PIN_5, DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_15},     // pin 45 OV7670_DAT5
	    { GPIOB,  GPIO_PIN_0, DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_8},      // pin 46 Green-LED
	    { GPIOB,  GPIO_PIN_1,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_9},     // pin 47
	    { GPIOB,  GPIO_PIN_2,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},            	 				// pin 48 SPI3-OUT
	    { GPIOF,  GPIO_PIN_11,  DIGITAL_IN | DIGITAL_OUT, NULL, 0},                            	// pin 49
	    { GPIOF,  GPIO_PIN_12,  DIGITAL_IN | DIGITAL_OUT, NULL, 0},                            	// pin 50
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 51 VSS
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 52 VDD
		{ GPIOF,  GPIO_PIN_13,  DIGITAL_IN | DIGITAL_OUT, NULL, 0},                            	// pin 53
		{ GPIOF,  GPIO_PIN_14,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            // pin 54
#endif
		{ GPIOF,  GPIO_PIN_15,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},           	 				// pin 55
	    { GPIOG,  GPIO_PIN_0,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},            	 				// pin 56
	    { GPIOG,  GPIO_PIN_1,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},            	 				// pin 57 SSD_RS
	    { GPIOE,  GPIO_PIN_7,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},            	 				// pin 58 SSD_D7
	    { GPIOE,  GPIO_PIN_8,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},            	 				// pin 59 SSD_D8
	    { GPIOE,  GPIO_PIN_9,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},            	 				// pin 60 SSD_D9
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 61 VSS
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 62 VDD
	    { GPIOE,  GPIO_PIN_10,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},           	 				// pin 63 SSD_D10
	    { GPIOE,  GPIO_PIN_11,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},           	 				// pin 64 SSD_D11
	    { GPIOE,  GPIO_PIN_12,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},           	 				// pin 65 SSD_D12
	    { GPIOE,  GPIO_PIN_13,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},           	 				// pin 66 SSD_D13
	    { GPIOE,  GPIO_PIN_14,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},           	 				// pin 67 SSD_D14
	    { GPIOE,  GPIO_PIN_15, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 68 SSD_D15
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 69 I2C2-SCL
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 70 I2C2-SDA
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 71 VCAP
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 72 VDD
	///
	    { GPIOB,  GPIO_PIN_12, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 73 COM3-RX (UART5)
	    { GPIOB,  GPIO_PIN_13, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 74 COM3-TX (UART5)
	    { GPIOB,  GPIO_PIN_14, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 75 Red-LED
	    { GPIOB,  GPIO_PIN_15, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                          	// pin 76 COM1-RX
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 77 CONSOLE-TX
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 78 CONSOLE-RX
	    { GPIOD,  GPIO_PIN_10, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 79
	    { GPIOD,  GPIO_PIN_11, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 80
	    { GPIOD,  GPIO_PIN_12, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 81 PWM-1A
	    { GPIOD,  GPIO_PIN_13, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 82 PWM-1B
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 83 VSS
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 84 VDD
		{ GPIOD,  GPIO_PIN_14, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 85 PWM-1C
		{ GPIOD,  GPIO_PIN_15, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 86 PWM-1D
		{ GPIOG,  GPIO_PIN_2, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 87 SD_CD
		{ GPIOG,  GPIO_PIN_3, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 88 OV7670_PCLK
		{ GPIOG,  GPIO_PIN_4, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 89 OV7670_VSYNC
		{ GPIOG,  GPIO_PIN_5, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 90 OV7670_HREF
		{ NULL,  0, PUNUSED , NULL, 0},                                                   	    // pin 91 USB-POWER
		{ GPIOG,  GPIO_PIN_7, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 92
		{ GPIOG,  GPIO_PIN_8, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 93 COUNT5-HIGHSPEED (H7 only)
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 94 VSS
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 95 VDDUSB
	    { GPIOC,  GPIO_PIN_6, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 96 OV7670_DAT6
	    { GPIOC,  GPIO_PIN_7, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 97 OV7670_DAT7
		{ GPIOC,  GPIO_PIN_8, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 98
		{ GPIOC,  GPIO_PIN_9, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 99
		{ GPIOA,  GPIO_PIN_8, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 100 OV7670_XCLK
		{ GPIOA,  GPIO_PIN_9, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 101
		{ GPIOA, GPIO_PIN_10, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 102
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 103 USB-D+
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 104 USB-D-
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 105 SWDIO
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 106 VCAP
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 107 VSS
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 108 VDD
	    //
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 109 SWCLK
		{ GPIOA,  GPIO_PIN_15, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 110
		{ GPIOC,  GPIO_PIN_10, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 111
		{ GPIOC,  GPIO_PIN_11, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 112
		{ GPIOC,  GPIO_PIN_12, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 113
	    { GPIOD,  GPIO_PIN_0, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 114
	    { GPIOD,  GPIO_PIN_1, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 115
		{ GPIOD,  GPIO_PIN_2, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 116
	    { GPIOD,  GPIO_PIN_3, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 117 SPI2-CLK
	    { GPIOD,  GPIO_PIN_4, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 118 COM2-DE
	    { GPIOD,  GPIO_PIN_5, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 119 COM2-TX
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 120 VSS
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 121 VDD
	    { GPIOD,  GPIO_PIN_6, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 122 COM2-RX
	    { GPIOD,  GPIO_PIN_7, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 123
	    { GPIOG,  GPIO_PIN_9, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 124 COM4-RX (USART6)
	    { GPIOG,  GPIO_PIN_10, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 125 SSD_WR
#ifdef STM32H753xx
	    { GPIOG,  GPIO_PIN_11, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 126 SPI-CLK
	    { GPIOG,  GPIO_PIN_12, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 127 SSD_RESET
	    { GPIOG,  GPIO_PIN_13, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 128 SSD_RD
#else
	    { GPIOG,  GPIO_PIN_11, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 126 
	    { GPIOG,  GPIO_PIN_12, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 127 SSD_RESET
	    { GPIOG,  GPIO_PIN_13, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 128 SSD_RD / SPI-CLK
#endif
	    { GPIOG,  GPIO_PIN_14, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 129 COM4-TX (USART6)
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 130 VSS
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 131 VDD
	    { GPIOG,  GPIO_PIN_15, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                           	// pin 132
	    { GPIOB,  GPIO_PIN_3, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 133 SPI3-CLK
	    { GPIOB,  GPIO_PIN_4, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 134 SPI3-IN
	    { GPIOB,  GPIO_PIN_5, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 135
	    { GPIOB,  GPIO_PIN_6, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 136 COM1-TX
		{ GPIOB,  GPIO_PIN_7, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 137 Blue-LED
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 138 BOOT0
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 139 I2C-SCL
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 140 I2C-SDA
	    { GPIOE,  GPIO_PIN_0, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 141 SSD_D0
	    { GPIOE,  GPIO_PIN_1, DIGITAL_IN | DIGITAL_OUT , NULL, 0},                            	// pin 142 SSD_D1
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 143 PDR-ON
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 144 VDD

};
#else
const struct s_PinDef PinDef64[NBR_PINS_64CHIP + 1]={
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 0
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 1 VBAT
		{ GPIOC,  GPIO_PIN_13,  DIGITAL_IN | DIGITAL_OUT,  NULL, 0},    						// pin 2 COUNT
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 3 OSC32_IN
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 4 OSC32_OUT
	    { GPIOH,  GPIO_PIN_0,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 5 COUNT
	    { GPIOH,  GPIO_PIN_1,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 6 COUNT
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 7 NRST
		{ GPIOC,  GPIO_PIN_0,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_0},    	// pin 8
		{ GPIOC,  GPIO_PIN_1,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_1},    	// pin 9
		{ GPIOC,  GPIO_PIN_2,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_2},    	// pin 10
		{ GPIOC,  GPIO_PIN_3,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_3},    	// pin 11
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 12 VSS
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 13 VDD
		{ GPIOA,  GPIO_PIN_0,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC2, ADC_CHANNEL_0},    	// pin 14 COM1-TX
	    { GPIOA,  GPIO_PIN_1,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC2, ADC_CHANNEL_1},     // pin 15 COM1-RX
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 16 CONSOLE-TX
		//
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 17 CONSOLE-RX
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 18 VSS
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 19 VDD
	    { GPIOA,  GPIO_PIN_4,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_4},    	// pin 20 DAC-1
	    { GPIOA,  GPIO_PIN_5,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_5},     // pin 21 DAC-2
	    { GPIOA,  GPIO_PIN_6,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_6},     // pin 22 PWM-1A
	    { GPIOA,  GPIO_PIN_7,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_7},     // pin 23 PWM-1B
		{ GPIOC,  GPIO_PIN_4,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN ,  ADC2, ADC_CHANNEL_4},    // pin 24 IR
		{ GPIOC,  GPIO_PIN_5,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN ,  ADC2, ADC_CHANNEL_5},    // pin 25
	    { GPIOB,  GPIO_PIN_0,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_8},     // pin 26 PWM-1C
	    { GPIOB,  GPIO_PIN_1,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_9},     // pin 27 PWM-1D
		{ GPIOB,  GPIO_PIN_2,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    						    // pin 28 COUNT
	    { GPIOB,  GPIO_PIN_10,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    						// pin 29 COM2-TX
	    { GPIOB,  GPIO_PIN_11,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    						// pin 30 COM2-RX
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 31 VCAP
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 32 VDD
		//
	    { GPIOB,  GPIO_PIN_12,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    						// pin 33 SSD_RESET
	    { GPIOB,  GPIO_PIN_13,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    						// pin 34 SPI2-CLK
	    { GPIOB,  GPIO_PIN_14,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    						// pin 35 SPI2-IN
	    { GPIOB,  GPIO_PIN_15,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    						// pin 36 SPI2-OUT
		{ GPIOC,  GPIO_PIN_6,  DIGITAL_IN | DIGITAL_OUT,  NULL, 0},    							// pin 37 COM3-TX
		{ GPIOC,  GPIO_PIN_7,  DIGITAL_IN | DIGITAL_OUT,  NULL, 0},    							// pin 38 COM3-RX
		{ GPIOC,  GPIO_PIN_8,  DIGITAL_IN | DIGITAL_OUT,  NULL, 0},    							// pin 39 SSD_WR
		{ GPIOC,  GPIO_PIN_9,  DIGITAL_IN | DIGITAL_OUT,  NULL, 0},    							// pin 40 SSD_RD
	    { GPIOA,  GPIO_PIN_8, DIGITAL_IN | DIGITAL_OUT , NULL, 0},      						// pin 41 SSD_D0
	    { GPIOA,  GPIO_PIN_9, DIGITAL_IN | DIGITAL_OUT , NULL, 0},      						// pin 42 SSD_D1
	    { GPIOA,  GPIO_PIN_10, DIGITAL_IN | DIGITAL_OUT , NULL, 0},      						// pin 43 SSD_D2
	    { GPIOA,  GPIO_PIN_11, DIGITAL_IN | DIGITAL_OUT , NULL, 0},      						// pin 44 SSD_D3
	    { GPIOA,  GPIO_PIN_12, DIGITAL_IN | DIGITAL_OUT , NULL, 0},      						// pin 45 SSD_D4
		{ GPIOA,  GPIO_PIN_13, DIGITAL_IN | DIGITAL_OUT , NULL, 0},      						// pin 46 SSD_D5/SWDIO
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 47 VCAP
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 48 VDD
		//
	    { GPIOA,  GPIO_PIN_14, DIGITAL_IN | DIGITAL_OUT , NULL, 0},      						// pin 49 SSD_D6/SWCLK
	    { GPIOA,  GPIO_PIN_15,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    						// pin 50 SSD_D7
		{ GPIOC,  GPIO_PIN_10,  DIGITAL_IN | DIGITAL_OUT,  NULL, 0},    						// pin 51 SPI3-CLK
		{ GPIOC,  GPIO_PIN_11,  DIGITAL_IN | DIGITAL_OUT,  NULL, 0},    						// pin 52 SPI3-IN
		{ GPIOC,  GPIO_PIN_12,  DIGITAL_IN | DIGITAL_OUT,  NULL, 0},    						// pin 53 SPI3-OUT
	    { GPIOD,  GPIO_PIN_2,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 54 SSD_RS
	    { GPIOB,  GPIO_PIN_3,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 55 SPI_CLK
	    { GPIOB,  GPIO_PIN_4,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 56 SPI_IN
	    { GPIOB,  GPIO_PIN_5,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 57 SPI-OUT
	    { GPIOB,  GPIO_PIN_6,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 58 I2C-SCL
	    { GPIOB,  GPIO_PIN_7,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 59 I2C-SDA
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 60 BOOT0
		{ GPIOB,  GPIO_PIN_8, DIGITAL_IN | DIGITAL_OUT , NULL, 0},      						// pin 61 PWM-2A
		{ GPIOB,  GPIO_PIN_9, DIGITAL_IN | DIGITAL_OUT , NULL, 0},      						// pin 62 PWM-2B
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 63 VSS
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 64 VDD
};

const struct s_PinDef PinDef100[NBR_PINS_100CHIP + 1]={
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 0
		{ GPIOE,  GPIO_PIN_2,  DIGITAL_IN | DIGITAL_OUT	,  NULL, 0},    						// pin 1 SSD_D2
		{ GPIOE,  GPIO_PIN_3,  DIGITAL_IN | DIGITAL_OUT	,  NULL, 0},    						// pin 2 SSD_D3
		{ GPIOE,  GPIO_PIN_4,  DIGITAL_IN | DIGITAL_OUT	,  NULL, 0},    						// pin 3 SSD_D4
		{ GPIOE,  GPIO_PIN_5,  DIGITAL_IN | DIGITAL_OUT ,  NULL, 0},    						// pin 4 SSD_D5
		{ GPIOE,  GPIO_PIN_6,  DIGITAL_IN | DIGITAL_OUT ,  NULL, 0},    						// pin 5 SSD_D6
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 6 VBAT
		{ GPIOC,  GPIO_PIN_13,  DIGITAL_IN | DIGITAL_OUT,  NULL, 0},    						// pin 7 COUNT
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 8 OSC32_IN
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 9 OSC32_OUT
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 10 VSS
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 11 VDD
	    { GPIOH,  GPIO_PIN_0,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 12 COUNT
	    { GPIOH,  GPIO_PIN_1,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 13 COUNT
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 14 NRST
		{ GPIOC,  GPIO_PIN_0,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_0},    	// pin 15
		{ GPIOC,  GPIO_PIN_1,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_1},    	// pin 16
		{ GPIOC,  GPIO_PIN_2,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_2},    	// pin 17
		{ GPIOC,  GPIO_PIN_3,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_3},    	// pin 18
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 19 VDD
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 20 VSSA
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 21 VREF+
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 22 VDDA
		{ GPIOA,  GPIO_PIN_0,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC2, ADC_CHANNEL_0},    	// pin 23 COM1-TX
	    { GPIOA,  GPIO_PIN_1,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC2, ADC_CHANNEL_1},     // pin 24 COM1-RX
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 25 CONSOLE-TX
		//
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 26 CONSOLE-RX
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 27 VSS
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 28 VDD
	    { GPIOA,  GPIO_PIN_4,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_4},    	// pin 29 DAC-1
	    { GPIOA,  GPIO_PIN_5,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_5},     // pin 30 DAC-2
	    { GPIOA,  GPIO_PIN_6,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_6},     // pin 31 PWM-1A
	    { GPIOA,  GPIO_PIN_7,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_7},     // pin 32 PWM-1B
		{ GPIOC,  GPIO_PIN_4,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN ,  ADC2, ADC_CHANNEL_4},    // pin 33 IR
		{ GPIOC,  GPIO_PIN_5,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN ,  ADC2, ADC_CHANNEL_5},    // pin 34
	    { GPIOB,  GPIO_PIN_0,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_8},     // pin 35 PWM-1C
	    { GPIOB,  GPIO_PIN_1,  DIGITAL_IN | DIGITAL_OUT | ANALOG_IN , ADC1, ADC_CHANNEL_9},     // pin 36 PWM-1D
		{ GPIOB,  GPIO_PIN_2,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    						    // pin 37 COUNT
		{ GPIOE,  GPIO_PIN_7,  DIGITAL_IN | DIGITAL_OUT	,  NULL, 0},    						// pin 38 SSD_D7
		{ GPIOE,  GPIO_PIN_8,  DIGITAL_IN | DIGITAL_OUT	,  NULL, 0},    						// pin 39 SSD_D8
		{ GPIOE,  GPIO_PIN_9,  DIGITAL_IN | DIGITAL_OUT	,  NULL, 0},    						// pin 40 SSD_D9
		{ GPIOE,  GPIO_PIN_10,  DIGITAL_IN | DIGITAL_OUT ,  NULL, 0},    						// pin 41 SSD_D10
		{ GPIOE,  GPIO_PIN_11,  DIGITAL_IN | DIGITAL_OUT ,  NULL, 0},    						// pin 42 SSD_D11
		{ GPIOE,  GPIO_PIN_12,  DIGITAL_IN | DIGITAL_OUT ,  NULL, 0},    						// pin 43 SSD_D12
		{ GPIOE,  GPIO_PIN_13,  DIGITAL_IN | DIGITAL_OUT ,  NULL, 0},    						// pin 44 SSD_D13
		{ GPIOE,  GPIO_PIN_14,  DIGITAL_IN | DIGITAL_OUT ,  NULL, 0},    						// pin 45 SSD_D14
		{ GPIOE,  GPIO_PIN_15,  DIGITAL_IN | DIGITAL_OUT ,  NULL, 0},    						// pin 46 SSD_D15
	    { GPIOB,  GPIO_PIN_10,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    						// pin 47 COM2-TX
	    { GPIOB,  GPIO_PIN_11,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    						// pin 48 COM2-RX
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 49 VCAP
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 50 VDD
		//
	    { GPIOB,  GPIO_PIN_12,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    						// pin 51 SSD_RESET
	    { GPIOB,  GPIO_PIN_13,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    						// pin 52 SPI2-CLK
	    { GPIOB,  GPIO_PIN_14,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    						// pin 53 SPI2-IN
	    { GPIOB,  GPIO_PIN_15,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    						// pin 54 SPI2-OUT
		{ GPIOD,  GPIO_PIN_8,  DIGITAL_IN | DIGITAL_OUT	,  NULL, 0},    						// pin 55
		{ GPIOD,  GPIO_PIN_9,  DIGITAL_IN | DIGITAL_OUT	,  NULL, 0},    						// pin 56
		{ GPIOD,  GPIO_PIN_10,  DIGITAL_IN | DIGITAL_OUT ,  NULL, 0},    						// pin 57
		{ GPIOD,  GPIO_PIN_11,  DIGITAL_IN | DIGITAL_OUT ,  NULL, 0},    						// pin 58
		{ GPIOD,  GPIO_PIN_12,  DIGITAL_IN | DIGITAL_OUT ,  NULL, 0},    						// pin 59
		{ GPIOD,  GPIO_PIN_13,  DIGITAL_IN | DIGITAL_OUT ,  NULL, 0},    						// pin 60
		{ GPIOD,  GPIO_PIN_14,  DIGITAL_IN | DIGITAL_OUT ,  NULL, 0},    						// pin 61
		{ GPIOD,  GPIO_PIN_15,  DIGITAL_IN | DIGITAL_OUT ,  NULL, 0},    						// pin 62
		{ GPIOC,  GPIO_PIN_6,  DIGITAL_IN | DIGITAL_OUT,  NULL, 0},    							// pin 63 COM3-TX
		{ GPIOC,  GPIO_PIN_7,  DIGITAL_IN | DIGITAL_OUT,  NULL, 0},    							// pin 64 COM3-RX
		{ GPIOC,  GPIO_PIN_8,  DIGITAL_IN | DIGITAL_OUT,  NULL, 0},    							// pin 65 SSD_WR
		{ GPIOC,  GPIO_PIN_9,  DIGITAL_IN | DIGITAL_OUT,  NULL, 0},    							// pin 66 SSD_RD
	    { GPIOA,  GPIO_PIN_8, DIGITAL_IN | DIGITAL_OUT , NULL, 0},      						// pin 67
	    { GPIOA,  GPIO_PIN_9, DIGITAL_IN | DIGITAL_OUT , NULL, 0},      						// pin 68
	    { GPIOA,  GPIO_PIN_10, DIGITAL_IN | DIGITAL_OUT , NULL, 0},      						// pin 69
	    { GPIOA,  GPIO_PIN_11, DIGITAL_IN | DIGITAL_OUT , NULL, 0},      						// pin 70
	    { GPIOA,  GPIO_PIN_12, DIGITAL_IN | DIGITAL_OUT , NULL, 0},      						// pin 71
		{ GPIOA,  GPIO_PIN_13, DIGITAL_IN | DIGITAL_OUT , NULL, 0},      						// pin 72 SWDIO
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 73 VCAP
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 74 VSS
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 75 VDD
		//
	    { GPIOA,  GPIO_PIN_14, DIGITAL_IN | DIGITAL_OUT , NULL, 0},      						// pin 76 SWCLK
	    { GPIOA,  GPIO_PIN_15,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    						// pin 77
		{ GPIOC,  GPIO_PIN_10,  DIGITAL_IN | DIGITAL_OUT,  NULL, 0},    						// pin 78 SPI3-CLK
		{ GPIOC,  GPIO_PIN_11,  DIGITAL_IN | DIGITAL_OUT,  NULL, 0},    						// pin 79 SPI3-IN
		{ GPIOC,  GPIO_PIN_12,  DIGITAL_IN | DIGITAL_OUT,  NULL, 0},    						// pin 80 SPI3-OUT
	    { GPIOD,  GPIO_PIN_0,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 81
	    { GPIOD,  GPIO_PIN_1,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 82
	    { GPIOD,  GPIO_PIN_2,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 83 SSD_RS
	    { GPIOD,  GPIO_PIN_3,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 84
	    { GPIOD,  GPIO_PIN_4,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 85
	    { GPIOD,  GPIO_PIN_5,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 86
	    { GPIOD,  GPIO_PIN_6,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 87
	    { GPIOD,  GPIO_PIN_7,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 88
	    { GPIOB,  GPIO_PIN_3,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 89 SPI_CLK
	    { GPIOB,  GPIO_PIN_4,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 90 SPI_IN
	    { GPIOB,  GPIO_PIN_5,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 91 SPI-OUT
	    { GPIOB,  GPIO_PIN_6,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 92 I2C-SCL
	    { GPIOB,  GPIO_PIN_7,  DIGITAL_IN | DIGITAL_OUT , NULL, 0},    							// pin 93 I2C-SDA
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 94 BOOT0
		{ GPIOB,  GPIO_PIN_8, DIGITAL_IN | DIGITAL_OUT , NULL, 0},      						// pin 95 PWM-2A
		{ GPIOB,  GPIO_PIN_9, DIGITAL_IN | DIGITAL_OUT , NULL, 0},      						// pin 96 PWM-2B
		{ GPIOE,  GPIO_PIN_0,  DIGITAL_IN | DIGITAL_OUT	,  NULL, 0},    						// pin 97 SSD_D0
		{ GPIOE,  GPIO_PIN_1,  DIGITAL_IN | DIGITAL_OUT	,  NULL, 0},    						// pin 98 SSD_D1
		{ NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 99 VSS
	    { NULL,  0, PUNUSED , NULL, 0},                                                       	// pin 100 VDD
};
#endif

#else
	const extern struct s_PinDef PinDef144[];
#endif      // DEFINE_PINDEF_TABLE
    extern struct s_PinDef *PinDef;
#ifndef STM32F4port
// Define the counting pin numbers
// INT1PIN refers to the PIC32 external interrupt #1, an so on for the others
#define INT1PIN              10 //PF0
#define INT2PIN              11 //PF1
#define INT3PIN              12 //PF2
#define INT4PIN              13 //PF3
#define IRPIN                14 //PF4
#ifdef STM32H753xx
	#define COUNT5				 93 //PG8
#endif
// I2C pin numbers
#define P_I2C_SCL           139 //PB8
#define P_I2C_SDA           140 //PB9
#define P_I2C2_SCL             69 //PB10
#define P_I2C2_SDA             70 //PB11

    // Console pin numbers
#define CONSOLE_RX_PIN		78 //PD9 USART3
#define CONSOLE_TX_PIN		77 //PD8
// COMx: port pin numbers
#define COM1_TX_PIN         136 //PB6 USART1
#define COM1_RX_PIN         76  //PB15

#define COM2_TX_PIN         119 //PD5 USART2
#define COM2_RX_PIN         122 //PD6
#define COM2_EN_PIN			118 //PD4

#define COM3_TX_PIN         74  //PB13 UART5
#define COM3_RX_PIN         73  //PB12

#define COM4_TX_PIN         129 //PG14 USART6
#define COM4_RX_PIN         124 //PG9

// SPI pin numbers
#define SPI_INP_PIN         42  //PA6
#define SPI_OUT_PIN         43  //PA7
#ifdef STM32H753xx
	#define SPI_CLK_PIN         126 //PG11
#else
	#define SPI_CLK_PIN         128 //PG13
#endif
// SPI2 pin numbers
#define SPI2_INP_PIN        28  //PC2
#define SPI2_OUT_PIN        27  //PC1
#define SPI2_CLK_PIN        117 //PD3
//
// SPI3 pin numbers
#define SPI3_CLK_PIN        133 //PB3
#define SPI3_INP_PIN        134 //PB4
#define SPI3_OUT_PIN        48  //PB2
// TOUCH pin numbers
#define SPI5_CLK_PIN        19  //PF7
#define SPI5_INP_PIN        20  //PF8
#define SPI5_OUT_PIN        21  //PF9
// DAC pin numbers
#define DAC_1_PIN           40  //PA4
#define DAC_2_PIN           41  //PA5
//
// PWM pin numbers
#define PWM_CH1_PIN         81 //PD12  - PWM1A
#define PWM_CH2_PIN         82 //PD13  - PWM1B
#define PWM_CH3_PIN         85 //PD14  - PWM1C
#define PWM_CH4_PIN         86 //PD15  - PWM1D
#define PWM_CH5_PIN         34 //PA0   - PWM2A
#define PWM_CH6_PIN         35 //PA1   - PWM2B
#define PWM_CH7_PIN         36 //PA2   - PWM2C
#define PWM_CH8_PIN         37 //PA3   - PWM2D

#define SSD1963_DAT1        141  	//E0
#define SSD1963_DAT2        142  	//E1
#define SSD1963_DAT3        1  		//E2
#define SSD1963_DAT4        2  		//E3
#define SSD1963_DAT5        3  		//E4
#define SSD1963_DAT6        4 		//E5
#define SSD1963_DAT7        5  		//E6
#define SSD1963_DAT8        58  	//E7
#define SSD1963_DAT9        59  	//E8
#define SSD1963_DAT10       60		//E9
#define SSD1963_DAT11       63  	//E10
#define SSD1963_DAT12       64  	//E11
#define SSD1963_DAT13       65  	//E12
#define SSD1963_DAT14       66  	//E13
#define SSD1963_DAT15       67  	//E14
#define SSD1963_DAT16       68  	//E15
// the SSD1963 write pin
#define SSD1963_WR_PIN      125		 //PG10
#define SSD1963_WR_TOGGLE_PIN   {GPIOG->ODR &= (~GPIO_PIN_10);GPIOG->ODR |= GPIO_PIN_10;}
#define SSD1963_WR_LOW   GPIOG->ODR &= (~GPIO_PIN_10);
#define SSD1963_WR_HIGH  GPIOG->ODR |= GPIO_PIN_10;
#ifdef STM32H753xx
	#define SSD1963_WR_TOGGLE_PIN_FAST   {GPIOG->BSRRH= GPIO_PIN_10;GPIOG->BSRRL= GPIO_PIN_10;}
#else
	#define SSD1963_WR_TOGGLE_PIN_FAST   {LL_GPIO_ResetOutputPin(GPIOG, GPIO_PIN_10);LL_GPIO_SetOutputPin(GPIOG,GPIO_PIN_10);}
#endif

// the SSD1963 data/command pin.  On the LCD it is labeled RS.
#define SSD1963_DC_PIN        57		//PG1
#define SSD1963_DC_LOW    GPIOG->ODR &= (~GPIO_PIN_1);
#define SSD1963_DC_HIGH   GPIOG->ODR |= GPIO_PIN_1;

// the SSD1963 reset pin
#define SSD1963_RESET_PIN   127		//PG12
#define SSD1963_RD_PIN    	128		//PG13
#define RDTOGGLE    GPIOG->ODR ^= GPIO_PIN_13;

// the OV7670 data pins
#define OV7670_DAT0        26  //PC0
#define OV7670_DAT1        27  //PC1
#define OV7670_DAT2        28  //PC2
#define OV7670_DAT3        29  //PC3
#define OV7670_DAT4        44  //PC4
#define OV7670_DAT5        45  //PC5
#define OV7670_DAT6        96  //PC6
#define OV7670_DAT7        97  //PC7
#define OV7670_PCLK        88  //PG3
#define OV7670_VSYNC       89  //PG4
#define OV7670_HREF        90  //PG5
#define OV7670_XCLK       100  //PA8
#define ST_HREF (GPIOG->IDR & GPIO_PIN_5)
#define ST_VSYNC (GPIOG->IDR & GPIO_PIN_4)
#define ST_PCLK (GPIOG->IDR & GPIO_PIN_3)

#else
#define INT1PIN              (HAS_64PINS ? 5  : 12) //PH0
#define INT2PIN              (HAS_64PINS ? 6  : 13) //PH1
#define INT3PIN              (HAS_64PINS ? 28  : 37) //PB2
#define INT4PIN              (HAS_64PINS ? 2  : 7) //PC13
#define IRPIN                (HAS_64PINS ? 24  : 33) //PC4
// I2C pin numbers
#define P_I2C_SCL           (HAS_64PINS ? 58 :92)//PB6
#define P_I2C_SDA           (HAS_64PINS ? 59 :93)//PB7

    // Console pin numbers
#define CONSOLE_RX_PIN		(HAS_64PINS ? 16 : 25) //PA2 USART2
#define CONSOLE_TX_PIN		(HAS_64PINS ? 17 : 26) //PA3
// COMx: port pin numbers
#define COM1_TX_PIN         (HAS_64PINS ? 14 : 23) //PA0 UART4
#define COM1_RX_PIN         (HAS_64PINS ? 15 : 24) //PA1

#ifdef STM32L446xx
    #define COM2_TX_PIN         (HAS_64PINS ? 29 : 47) //PB10 USART3
	#define COM2_RX_PIN         (HAS_64PINS ? 25 : 48) //PC5
#else
	#define COM2_TX_PIN         (HAS_64PINS ? 29 : 47) //PB10 USART3
	#define COM2_RX_PIN         (HAS_64PINS ? 30 : 48) //PB11
#endif

#ifndef STM32L476xx
    #define COM3_TX_PIN         (HAS_64PINS ? 37 : 63) //PC6 USART6
	#define COM3_RX_PIN         (HAS_64PINS ? 38 : 64) //PC7
#endif
// SPI pin numbers
#define SPI_CLK_PIN         (HAS_64PINS ? 55 : 89) //PB3
#define SPI_INP_PIN         (HAS_64PINS ? 56 : 90) //PB4
#define SPI_OUT_PIN         (HAS_64PINS ? 57 : 91) //PB5

// SPI2 pin numbers
#define SPI2_CLK_PIN        (HAS_64PINS ? 34 : 52) //PB13
#define SPI2_INP_PIN        (HAS_64PINS ? 35 : 53) //PB14
#define SPI2_OUT_PIN        (HAS_64PINS ? 36 : 54) //PB15
//
// SPI3 pin numbers
#define SPI3_OUT_PIN        (HAS_64PINS ? 51 : 78) //PC10
#define SPI3_CLK_PIN        (HAS_64PINS ? 52 : 79) //PC11
#define SPI3_INP_PIN        (HAS_64PINS ? 53 : 80) //PC12

// DAC pin numbers
#define DAC_1_PIN           (HAS_64PINS ? 20 : 29)  //PA4
#define DAC_2_PIN           (HAS_64PINS ? 21 : 30)  //PA5
//
// PWM pin numbers
#define PWM_CH1_PIN         (HAS_64PINS ? 22 : 31) //PA6  - PWM1A
#define PWM_CH2_PIN         (HAS_64PINS ? 23 : 32) //PA7  - PWM1B
#define PWM_CH3_PIN         (HAS_64PINS ? 26 : 35) //PB0  - PWM1C
#define PWM_CH4_PIN         (HAS_64PINS ? 27 : 36) //PB1  - PWM1D
#define PWM_CH5_PIN         (HAS_64PINS ? 61 : 95) //PB8  - PWM2A
#define PWM_CH6_PIN         (HAS_64PINS ? 62 : 96) //PB9  - PWM2B

#define SSD1963_DAT1        (HAS_64PINS ? 41 : 97) 	//PA8, PE0
#define SSD1963_DAT2        (HAS_64PINS ? 42 : 98) 	//PA9, PE1
#define SSD1963_DAT3        (HAS_64PINS ? 43 : 1) 	//PA10, PE2
#define SSD1963_DAT4        (HAS_64PINS ? 44 : 2) 	//PA11, PE3
#define SSD1963_DAT5        (HAS_64PINS ? 45 : 3) 	//PA12, PE4
#define SSD1963_DAT6        (HAS_64PINS ? 46 : 4)	//PA13, PE5
#define SSD1963_DAT7        (HAS_64PINS ? 49 : 5) 	//PA14, PE6
#define SSD1963_DAT8        (HAS_64PINS ? 50 : 38) 	//PA15, PE7
#define SSD1963_DAT9        (HAS_64PINS ? 0 : 39)   //0, PE8
#define SSD1963_DAT10       (HAS_64PINS ? 0 : 40)   //0, PE9
#define SSD1963_DAT11       (HAS_64PINS ? 0 : 41)   //0, PE10
#define SSD1963_DAT12       (HAS_64PINS ? 0 : 42)   //0, PE11
#define SSD1963_DAT13       (HAS_64PINS ? 0 : 43)   //0, PE12
#define SSD1963_DAT14       (HAS_64PINS ? 0 : 44)   //0, PE13
#define SSD1963_DAT15       (HAS_64PINS ? 0 : 45)   //0, PE14
#define SSD1963_DAT16       (HAS_64PINS ? 0 : 46)   //0, PE15
// the SSD1963 write pin
#define SSD1963_WR_PIN      (HAS_64PINS ? 39 : 65)	//PC8
#define SSD1963_WR_TOGGLE_PIN   {GPIOB->ODR &= (~GPIO_PIN_8);GPIOB->ODR |= GPIO_PIN_8;}
#define SSD1963_WR_LOW   GPIOB->ODR &= (~GPIO_PIN_8);
#define SSD1963_WR_HIGH  GPIOB->ODR |= GPIO_PIN_8;
#define SSD1963_WR_TOGGLE_PIN_FAST   {LL_GPIO_ResetOutputPin(GPIOB, GPIO_PIN_8);LL_GPIO_SetOutputPin(GPIOB,GPIO_PIN_8);}


// the SSD1963 data/command pin.  On the LCD it is labeled RS.
#define SSD1963_DC_PIN        (HAS_64PINS ? 54 : 83) //PD2
#define SSD1963_DC_LOW    GPIOD->ODR &= (~GPIO_PIN_2);
#define SSD1963_DC_HIGH   GPIOD->ODR |= GPIO_PIN_2;

// the SSD1963 reset pin
#define SSD1963_RESET_PIN   (HAS_64PINS ? 33 : 51)  //PB12
#define SSD1963_RD_PIN    	(HAS_64PINS ? 40 : 66)	//PC9
#define RDTOGGLE    GPIOB->ODR ^= GPIO_PIN_9;
#endif
#endif

